<?php

namespace threewp_broadcast\premium_pack\code_snippets
{

/**
	@brief			Run code snippets on specific blogs.
	@plugin_group	Utilities
	@since			2021-09-29 20:27:59
**/
class Code_Snippets
	extends \threewp_broadcast\premium_pack\base
{
	public function _construct()
	{
		$this->add_action( 'threewp_broadcast_menu' );
		$this->run_snippets();
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Callbacks
	// --------------------------------------------------------------------------------------------

	/**
		@brief		Run all of the snippets for this blog.
		@since		2021-09-29 22:11:56
	**/
	public function run_snippets()
	{
		$snippets = $this->snippets();
		foreach( $snippets->get_for_this_blog() as $snippet )
			$snippet->run();
	}

	/**
		@brief		Add ourself to Broadcast's menu.
		@since		2021-09-29 21:48:43
	**/
	public function threewp_broadcast_menu( $action )
	{
		$action->menu_page
			->submenu( 'threewp_broadcast_code_snippets' )
			->callback_this( 'admin_menu_tabs' )
			->menu_title( 'Code Snippets' )
			->page_title( 'Broadcast Code Snippets' );
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Admin
	// --------------------------------------------------------------------------------------------

	/**
		@brief		Edit a snippet.
		@since		2021-07-18 22:02:59
	**/
	public function admin_edit_snippet( $snippet_id )
	{
		$snippets = $this->snippets();
		$snippet = $snippets->get( $snippet_id );

		if ( ! $snippet )
			wp_die( 'Invalid snippet!' );

		$form = $this->form();
		$r = '';

		$enabled_input = $form->checkbox( 'enabled' )
			->checked( $snippet->is_enabled() )
			->description( __( 'Is this snippet enabled at all?', 'threewp_broadcast' ) )
			->label( __( 'Enabled', 'threewp_broadcast' ) );

		$description_input = $form->text( 'description' )
			->description( __( 'Describe this snippet for yourself.', 'threewp_broadcast' ) )
			->label( __( 'Description', 'threewp_broadcast' ) )
			->required()
			->size( 40 )
			->value( $snippet->get( 'description' ) );

		$blogs_select = $this->add_blog_list_input( [
			// Blog selection input description
			'description' => __( 'Select one or more blogs on which to run the code.', 'threewp_broadcast' ),
			'form' => $form,
			// Blog selection input label
			'label' => __( 'Blogs', 'threewp_broadcast' ),
			'multiple' => true,
			'name' => 'blogs',
			'required' => false,
			'value' => $snippet->get_blogs()->to_array(),
		] );

		$code_input = $form->textarea( 'code' )
			->description( __( 'The PHP code snippet to be run.', 'threewp_broadcast' ) )
			->label( __( 'PHP code', 'threewp_broadcast' ) )
			->rows( 20, 80 )
			->value( $snippet->get( 'code' ) )
			->stripslashes();

		$save_button = $form->primary_button( 'save' )
			->value( __( 'Save', 'threewp_broadcast' ) );

		if ( $form->is_posting() )
		{
			$form->post();
			$form->use_post_values();

			$snippet->set_blogs( $blogs_select->get_post_value() );
			$snippet->set( 'code', $code_input->get_post_value() );
			$snippet->set( 'description', $description_input->get_post_value() );
			$snippet->set( 'enabled', $enabled_input->is_checked() );
			$snippets->save();

			$r .= $this->info_message_box()
				->_( __( 'The settings for this snippet have been saved!', 'threewp_broadcast' ) );
		}

		$r .= $form->open_tag();
		$r .= $form->display_form_table();
		$r .= $form->close_tag();

		echo $r;
	}

	/**
		@brief		admin_menu_overview
		@since		2021-09-29 21:37:19
	**/
	public function admin_menu_overview()
	{
		$form = $this->form();
		$r = '';
		$table = $this->table();

		$row = $table->head()->row();
		$table->bulk_actions()
			->form( $form )
			->add( __( 'Delete', 'threewp_broadcast' ), 'delete' )
			->cb( $row );

		$row->th( 'enabled' )->text( __( 'Enabled', 'threewp_broadcast' ) );
		$row->th( 'blogs' )->text( __( 'Blogs', 'threewp_broadcast' ) );
		$row->th( 'description' )->text( __( 'Description', 'threewp_broadcast' ) );

		$snippets = $this->snippets();

		$fs = $form->fieldset( 'fs_create_new' );
		$fs->legend->label( 'Create a new code snippet' );

		$button_create_code_snippet = $form->primary_button( 'create_code_snippet' )
			// Create snippet button
			->value( __( 'Create a new snippet', 'threewp_broadcast' ) );

		if ( $form->is_posting() )
		{
			$form->post();
			if ( $table->bulk_actions()->pressed() )
			{
				switch ( $table->bulk_actions()->get_action() )
				{
					case 'delete':
						$ids = $table->bulk_actions()->get_rows();

						foreach( $ids as $id )
							$snippets->forget( $id );

						$r .= $this->info_message_box()
							->_( __( 'The selected rows have been deleted.', 'threewp_broadcast' ) );
						$snippets->save();
					break;
				}
			}

			if ( $button_create_code_snippet->pressed() )
			{
				$snippet = Snippet::create();
				$snippets->append( $snippet );
				$this->message( sprintf(
					__( 'A new code snippet, %s, has been created.', 'threewp_broadcast' ),
					'<em>' . $snippet->get( 'description' ) . '</em>'
				) );
				$snippets->save();
			}
		}

		foreach( $snippets as $snippet )
		{
			$row = $table->body()->row();
			$snippet_id = $snippet->get( 'id' );
			$table->bulk_actions()->cb( $row, $snippet_id );

			$edit_url = add_query_arg( 'snippet_id', $snippet_id );
			$edit_url = add_query_arg( 'tab', 'edit', $edit_url );

			$description = sprintf( '<a href="%s" title="%s">%s</a>',
				$edit_url,
				__( 'Edit this snippet', 'threewp_broadcast' ),
				$snippet->get( 'description' )
			);

			$enabled = $snippet->is_enabled() ? 'Yes' : 'No';
			$row->td( 'enabled' )->text( $enabled );
			$row->td( 'blogs' )->text( count( $snippet->get_blogs() ) );
			$row->td( 'description' )->text( $description );
		}

		$r .= $this->p( __( 'This add-on runs code snippets selectively.', 'threewp_broadcast' ) );

		$r .= $form->open_tag();
		$r .= $table;
		$r .= $form->display_form_table();
		$r .= $form->close_tag();

		echo $r;
	}

	/**
		@brief		admin_menu_tabs
		@since		2021-09-29 21:37:19
	**/
	public function admin_menu_tabs()
	{
		$tabs = $this->tabs();

		$tabs->tab( 'overview' )
			->callback_this( 'admin_menu_overview' )
			->heading( 'Broadcast Code Snippets overview' )
			// Tab name
			->name( 'Overview' )
			->sort_order( 25 );

		if ( $tabs->get_is( 'edit' ) )
		{
			$id = $_GET[ 'snippet_id' ];
			$id = intval( $id );
			$snippets = $this->snippets();
			$snippet = $snippets->get( $id );

			$tabs->tab( 'edit' )
				->callback_this( 'admin_edit_snippet' )
				// Tab name
				->heading_( __( 'Edit snippet %s', 'threewp_broadcast' ), $snippet->get( 'description' ) )
				->name( __( 'Edit snippet', 'threewp_broadcast' ) )
				->parameters( $id );
		}

		echo $tabs->render();
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Misc functions
	// --------------------------------------------------------------------------------------------

	/**
		@brief		Loader for the snippet container.
		@since		2021-09-29 20:29:23
	**/
	public function snippets()
	{
		return Snippets::load();
	}
}

} // namespace

namespace
{
	function Broadcast_Code_Snippets()
	{
		return threewp_broadcast\premium_pack\code_snippets\Code_Snippets::instance();
	}
}
