<?php
namespace threewp_broadcast\premium_pack\cron_everywhere;

/**
	@brief			Runs WordPress cron periodically on all sites of the network.
	@plugin_group	Utilities
	@since			2021-09-23 14:03:12
**/
class Cron_Everywhere
	extends \threewp_broadcast\premium_pack\base
{
	/**
		@brief		The name of the action we are listening for.
		@since		2021-09-23 14:04:29
	**/
	public static $action_name = 'broadcast_cron_everywhere_now';

	/**
		@brief		The transient key for when we are busy.
		@since		2021-09-26 22:15:37
	**/
	public static $busy_transient_key;

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Admin
	// --------------------------------------------------------------------------------------------

	public function _construct()
	{
		$this->add_action( static::$action_name );
	}

	/**
		@brief		Activate.
		@since		2021-09-26 22:20:20
	**/
	public function activate()
	{
		$this->reschedule();
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Misc
	// --------------------------------------------------------------------------------------------

	/**
		@brief		broadcast_cron_everywhere_now
		@since		2021-09-23 14:07:42
	**/
	public function broadcast_cron_everywhere_now()
	{
		$this->reschedule();

		// Are we busy?
		if ( $this->is_busy() )
			return $this->debug( 'Still busy.' );

		$this->debug( 'Running from %s', get_current_blog_id() );

		$this->set_busy();

		global $wpdb;
		$sql = sprintf( "SELECT `domain`, `path` FROM `%s` WHERE `archived` = '0' AND `deleted` ='0'",
			$wpdb->blogs
		);
		$blogs = $wpdb->get_results( $sql );
		$this->debug( 'Running on %s blogs.', count( $blogs ) );

		foreach( $blogs as $blog )
		{
			$command = ( is_ssl() ? 'https://' : 'http://' ) . $blog->domain . ($blog->path ? $blog->path : '/') . 'wp-cron.php';
			$this->debug( 'Running %s', $command );
			$ch = curl_init($command);
			$rc = curl_setopt($ch, CURLOPT_RETURNTRANSFER, FALSE);
			$rc = curl_exec($ch);
			curl_close($ch);
		}

		$this->debug( 'Not busy anymore.' );
		$this->clear_busy();
	}

	/**
		@brief		Are we busy with a cron?
		@since		2021-09-26 22:16:03
	**/
	public function is_busy()
	{
		$value = get_site_transient( static::$busy_transient_key );
		if ( ! $value )
			return false;
		return true;
	}

	/**
		@brief		Reschedule a cron call.
		@since		2021-09-23 14:04:03
	**/
	public function reschedule()
	{
		$this->debug( 'Rescheduling on blog %s.', get_current_blog_id() );
		wp_clear_scheduled_hook( static::$action_name );
		wp_schedule_single_event( time() + 60, static::$action_name );
	}

	/**
		@brief		Mark us as busy.
		@since		2021-09-26 22:16:38
	**/
	public function set_busy()
	{
		$this->debug( 'We are now busy.' );
		set_site_transient( static::$busy_transient_key, true, HOUR_IN_SECONDS );
	}

	/**
		@brief		Clear the busy status.
		@since		2021-09-26 22:17:29
	**/
	public function clear_busy()
	{
		delete_site_transient( static::$busy_transient_key );
	}

}
