<?php

namespace threewp_broadcast\premium_pack\republisher
{

/**
	@brief			Automatically republish posts on a schedule.
	@plugin_group	Control
	@since			2021-08-22 21:26:02
**/
class Republisher
	extends \threewp_broadcast\premium_pack\base
{
	/**
		@brief		The meta key that specifies how often to republish the child posts.
		@since		2021-08-22 21:38:28
	**/
	public static $republish_hours_meta_key = 'broadcast_republish_hours';

	public function _construct()
	{
		$this->add_action( 'broadcast_republish_post' );
		$this->add_action( 'threewp_broadcast_broadcasting_before_restore_current_blog' );
		$this->add_action( 'threewp_broadcast_broadcasting_started' );
		$this->add_action( 'threewp_broadcast_prepare_meta_box' );
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Admin
	// --------------------------------------------------------------------------------------------

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Callbacks
	// --------------------------------------------------------------------------------------------

	/**
		@brief		Republish this post now.
		@since		2021-08-22 22:14:22
	**/
	public function broadcast_republish_post( $post_id )
	{
		$this->debug( 'Going to republish post %s on blog %s.', $post_id, get_current_blog_id() );

		$gmt_offset = current_time( 'timestamp' ) - current_time( 'timestamp', true );
		$new_post_time = time();

		$new_post_data = [
			'ID' => $post_id,
			'post_date' => date( 'Y-m-d H:i:s', $new_post_time + $gmt_offset ),
			'post_date_gmt' => date( 'Y-m-d H:i:s', $new_post_time ),
		];

		$this->debug( 'New post data: %s', $new_post_data );

		wp_update_post( $new_post_data );

		$this->maybe_schedule_republish( $post_id );

		$this->debug( 'Done republishing post %s.', $post_id );
		exit;
	}

	/**
		@brief		Schedule a republish?
		@since		2021-08-22 22:06:38
	**/
	public function threewp_broadcast_broadcasting_before_restore_current_blog( $action )
	{
		$bcd = $action->broadcasting_data;

		if ( ! isset( $bcd->republish_hours ) )
			return;

		$this->maybe_schedule_republish( $bcd->new_post( 'ID' ) );
	}

	/**
		@brief		Started.
		@since		2021-07-20 22:51:06
	**/
	public function threewp_broadcast_broadcasting_started( $action )
	{
		$bcd = $action->broadcasting_data;
		$mbd = $bcd->meta_box_data;

		$input = $mbd->form->input( 'broadcast_republish_hours' );
		if ( ! $input )
			return;

		$republish_hours = $input->get_filtered_post_value();

		if ( $republish_hours < 1 )
			return;

		$this->debug( 'Child posts will be republished every %s hours.', $republish_hours );
		$bcd->republish_hours = $republish_hours;

		$bcd->custom_fields()->set( static::$republish_hours_meta_key, $republish_hours );
	}

	/**
		@brief		Modify the meta box with modification data.
		@since		20131016
	**/
	public function threewp_broadcast_prepare_meta_box( $action )
	{
		$meta_box_data = $action->meta_box_data;
		$form = $meta_box_data->form;
		$post = $action->meta_box_data->post;

		$republish_hours = get_post_meta( $post->ID, static::$republish_hours_meta_key, true );
		$republish_hours = intval( $republish_hours );

		$broadcast_republish_hours_input = $form->number( 'broadcast_republish_hours' )
			->description( 'How often to republish the child posts.' )
			->label( 'Republish hours' )
			->min( 0 )
			->value( $republish_hours );
		$meta_box_data->convert_form_input_later( 'broadcast_republish_hours' );
	}

	// --------------------------------------------------------------------------------------------
	// ----------------------------------------- Misc functions
	// --------------------------------------------------------------------------------------------

	/**
		@brief		Schedule a republish of this post.
		@since		2021-08-22 22:07:57
	**/
	public function maybe_schedule_republish( $post_id )
	{
		$post = get_post( $post_id );
		$republish_hours = get_post_meta( $post_id, static::$republish_hours_meta_key, true );

		if ( $republish_hours < 1 )
		{
			$this->debug( 'Post %s does not want to be republished.' );
			wp_clear_scheduled_hook( 'broadcast_republish_post', [ $post_id ] );
			return;
		}

		$next_time = strtotime( $post->post_date_gmt ) + ( $republish_hours * HOUR_IN_SECONDS );

		$this->debug( 'Scheduling a republish in %s (GMT) + %d hours = %s',
			$post->post_date_gmt,
			$republish_hours,
			date( 'Y-m-d H:i', $next_time )
		);
		wp_schedule_single_event( $next_time, 'broadcast_republish_post', [ $post_id ] );
	}
}

} // namespace

namespace
{
	function Broadcast_Republisher()
	{
		return threewp_broadcast\premium_pack\republisher\Republisher::instance();
	}
}
